/*
 * Copyright 2020 Ladifire. All rights reserved.
 * This file is licensed to you under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License. You may obtain a copy
 * of the License at http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under
 * the License is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR REPRESENTATIONS
 * OF ANY KIND, either express or implied. See the License for the specific language
 * governing permissions and limitations under the License.
 */

import * as React from 'react';

import {useFocusableRef, useStyleProps, joinClasses} from '@ladifire-ui-react/utils';
import {FocusableRef} from '@react-ui-types/shared';
import {FocusRing} from '@react-ui-aria/focus';
import {SpectrumCheckboxProps} from '@react-ui-types/checkbox';
import {useCheckbox} from '@react-aria/checkbox';
import {useHover} from '@react-ui-aria/interactions';
import {useProviderProps} from '@ladifire-ui-react/provider';
import {useToggleState} from '@react-ui-stately/toggle';
import {CheckMarkIcon} from '@ladifire-ui-react/icons';
import {CommonIcons} from '@ladifire-ui-react/icons';

import stylex from '@ladifire/stylex';

type Style =
  | 'dash'
  | 'label'
  | 'checkbox'
  | 'checkboxChecked'

const styles: any = stylex.create<Style>({
  dash: {
    marginLeft: 1,
  },
  label: {
    display: 'inline-flex',
    boxSizing: 'border-box',
    alignItems: 'flex-start',
    position: 'relative',
    verticalAlign: 'top',
    maxWidth: '100%',
    isolation: 'isolate',
  },
  checkbox: {
    '::before': {
      position: 'absolute',
      content: '""',
      zIndex: 0,
      boxSizing: 'border-box',
      width: 18,
      height: 18,
      borderRadius: 4,
      borderWidth: 2,
      borderStyle: 'solid',
      borderColor: 'var(--always-gray-75)',
      transition: 'border 100ms ease-in-out, box-shadow 100ms ease-in-out',
    },
    '::after': {
      position: 'absolute',
      content: '""',
      display: 'block',
      boxSizing: 'border-box',
      borderRadius: 1,
      left: 0,
      right: 0,
      bottom: 0,
      top: 0,
      margin: 2,
      transform: 'translateX(0)',
      transition: 'opacity 100ms ease-out, margin 100ms ease-out',
    },
  },
  checkboxChecked: {
    '::before': {
      borderWidth: 9,
      borderColor: 'var(--accent)',
    },
    '::after': {
      a: 1,
      color: 'red'
    },
  },
});

function Checkbox(props: SpectrumCheckboxProps, ref: FocusableRef<HTMLLabelElement>) {
  props = useProviderProps(props);
  let {
    isIndeterminate = false,
    isEmphasized = false,
    isDisabled = false,
    autoFocus,
    children,
    disabledHoverBackground = false,
    ...otherProps
  } = props;
  let {styleProps} = useStyleProps(otherProps);
  let {hoverProps, isHovered} = useHover(props);

  let inputRef = React.useRef<HTMLInputElement>(null);
  let domRef = useFocusableRef(ref, inputRef);
  let state = useToggleState(props);
  let {inputProps} = useCheckbox(props, state, inputRef);

  let markIcon = isIndeterminate
    ? (
      <CommonIcons.IDash16
        filter="secondary-icon"
        UNSAFE_className={stylex(styles.dash)}
      />
    )
    : (
      <CheckMarkIcon
        size={18}
        UNSAFE_className={stylex.dedupe(
          {
            display: 'inline-block',
            pointerEvents: 'none',
            fill: 'currentColor',
            width: 18,
            height: 18,
            position: 'absolute',
            color: 'var(--always-white)',
            transition: 'opacity 100ms ease-in-out, transform 100ms ease-in-out',
          },
          state.isSelected ? {
            opacity: 1,
          } : {
            opacity: 0,
          },
        )}
      />
    );

  return (
    <label
      {...styleProps}
      {...hoverProps}
      ref={domRef}
      className={joinClasses(
        stylex(styles.label),
        styleProps.className,
      )}
    >
      <FocusRing focusClass='focus-ring' autoFocus={autoFocus}>
        <input
          {...inputProps}
          ref={inputRef}
          className={
            stylex.dedupe(
              {
                margin: 0,
                overflow: 'visible',
                boxSizing: 'border-box',
                padding: 0,
                position: 'absolute',
                top: 0,
                height: '100%',
                zIndex: 1,
                cursor: 'pointer',
                opacity: 0,
                width: '100%',
              },
              isDisabled ? {
                pointerEvents: 'none',
              } : null,
            )
          } />
      </FocusRing>
      <span
        className={joinClasses(stylex(styles.checkbox), stylex.dedupe(
          {
            position: 'relative',
            boxSizing: 'border-box',
            flexGrow: 1,
            height: 18,
            width: 18,
          },
          isDisabled ? {
            opacity: 0.5,
          } : null,
        ), state.isSelected && stylex(styles.checkboxChecked))}
      >
        {markIcon}
        {
          !disabledHoverBackground && (
            <div
              className={stylex.dedupe(
                {
                  position: 'absolute',
                  top: 0,
                  left: 0,
                  bottom: 0,
                  right: 0,
                  pointerEvents: 'none',
                  transitionTimingFunction: 'var(--lf-animation-fade-out)',
                  transitionDuration: 'var(--lf-duration-extra-extra-short-out)',
                  transitionProperty: 'opacity',
                  cursor: 'pointer',
                  borderRadius: '50%',
                  backgroundColor: 'var(--press-overlay)',
                  marginLeft: -8,
                  marginRight: -8,
                  marginTop: -8,
                  marginBottom: -8,
                },
                isHovered ? {
                  opacity: 1,
                } : {
                  opacity: 0,
                },
              )}
            />
          )
        }
      </span>
      {children && (
        <span
          className={stylex.dedupe(
            {
              textAlign: 'start',
              marginLeft: 12,
            },
            isDisabled ? {
              opacity: 0.5
            } : null,
          )}
        >
          {children}
        </span>
      )}
    </label>
  );
}
/**
 * Checkboxes allow users to select multiple items from a list of individual items,
 * or to mark one individual item as selected.
 */
let _Checkbox = React.forwardRef(Checkbox);
export {_Checkbox as Checkbox};
